LibStub:GetLibrary("AceComm-3.0"):Embed(Recount)
LibStub:GetLibrary("AceSerializer-3.0"):Embed(Recount)

local revision = tonumber(string.sub("$Revision: 68815 $", 12, -3))
if Recount.Version < revision then Recount.Version = revision end

local MinimumVersion=67644

-- Elsia: This is straight from GUIDRegistryLib-0.1 by ArrowMaster.

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_AFFILIATION_OUTSIDER	= COMBATLOG_OBJECT_AFFILIATION_OUTSIDER	or 0x00000008
-- Reaction
local COMBATLOG_OBJECT_REACTION_FRIENDLY	= COMBATLOG_OBJECT_REACTION_FRIENDLY	or 0x00000010
-- Ownership
local COMBATLOG_OBJECT_CONTROL_PLAYER		= COMBATLOG_OBJECT_CONTROL_PLAYER		or 0x00000100
-- Unit type
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local COMBATLOG_OBJECT_TYPE_GUARDIAN		= COMBATLOG_OBJECT_TYPE_GUARDIAN		or 0x00002000

-- Setting up some useful flag combos to bootstrap synced combatants that need to be added.
local PARTY_GUARDIAN_FLAGS = COMBATLOG_OBJECT_AFFILIATION_RAID + COMBATLOG_OBJECT_REACTION_FRIENDLY + COMBATLOG_OBJECT_CONTROL_PLAYER + COMBATLOG_OBJECT_TYPE_GUARDIAN
local PARTY_PET_FLAGS = COMBATLOG_OBJECT_AFFILIATION_RAID + COMBATLOG_OBJECT_REACTION_FRIENDLY + COMBATLOG_OBJECT_CONTROL_PLAYER + COMBATLOG_OBJECT_TYPE_PET
local PARTY_GUARDIAN_OWNER_FLAGS = COMBATLOG_OBJECT_AFFILIATION_RAID + COMBATLOG_OBJECT_REACTION_FRIENDLY + COMBATLOG_OBJECT_CONTROL_PLAYER + COMBATLOG_OBJECT_TYPE_PLAYER


-- Elsia: Generic Sync code here

function Recount:CheckVisible()

	local _ , instanceType = IsInInstance()
	
	if instanceType == "pvp" then return end

	if GetNumRaidMembers() > 0 then
		for i = 1, 40, 1 do 
			local unitid = "raid"..i
			if not UnitIsVisible(unitid) and UnitExists(unitid) then
				local name = UnitName(unitid)
				local combatant = Recount.db2.combatants[name]
			
				if not combatant then
					Recount:AddCombatant(name,nil,UnitGUID(unitid),COMBATLOG_OBJECT_AFFILIATION_RAID+COMBATLOG_OBJECT_TYPE_PLAYER+COMBATLOG_OBJECT_REACTION_FRIENDLY,nil)
					combatant = Recount.db2.combatants[name]
				end
			
				Recount.db2.combatants[name].lazysync = true
				Recount.lazysync = Recount.lazysync or Recount:CheckRetention(name)
			end
		end
	elseif GetNumPartyMembers() > 0 then
		for i = 1, 4, 1 do 
			local unitid = "party"..i
			if not UnitIsVisible(unitid) and UnitExists(unitid) then
				local name = UnitName(unitid)
				local combatant = Recount.db2.combatants[name]
			
				if not combatant then
					Recount:AddCombatant(name,nil,UnitGUID(unitid),COMBATLOG_OBJECT_AFFILIATION_PARTY+COMBATLOG_OBJECT_TYPE_PLAYER+COMBATLOG_OBJECT_REACTION_FRIENDLY,nil)
					combatant = Recount.db2.combatants[name]
				end
			
				Recount.db2.combatants[name].lazysync = true
				Recount.lazysync = Recount.lazysync or Recount:CheckRetention(name)
			end
		end
	end
end

function Recount:SendSelf(target)
	-- Prepare self
	local myname = Recount.PlayerName
	local combatant = Recount.db2.combatants[myname]
	
	if not combatant then
		return
	end
	
	local myrecord = combatant.Sync
	
	if not myrecord then
		return
	end
	
	local serialdata
	local damage = myrecord.Damage or 0
	local damagetaken = myrecord.DamageTaken or 0
	local healing = myrecord.Healing or 0
	local overhealing = myrecord.Overhealing or 0
	local healingtaken = myrecord.HealingTaken or 0
	local activetime = myrecord.ActiveTime or 0
	serialdata = Recount:Serialize("PS",myname,damage,damagetaken,healing,overhealing,healingtaken,activetime)
	
	-- Prepare Pets
	local data=combatant	
	local serialpetdata = {}
	
	if data.Pet then
		for i=1,#data.Pet do
			local petrecord = Recount.db2.combatants[data.Pet[i]] and Recount.db2.combatants[data.Pet[i]].Sync
			if petrecord then
				local damage = petrecord.Damage or 0
				local damagetaken = petrecord.DamageTaken or 0
				local healing = petrecord.Healing or 0
				local overhealing = petrecord.Overhealing or 0
				local healingtaken = petrecord.HealingTaken or 0
				local activetime = petrecord.ActiveTime or 0
				if petrecord and petrecord.Touched and damage + damagetaken + healing + overhealing + healingtaken ~= 0 then
					tinsert(serialpetdata, Recount:Serialize("PS",data.Pet[i],damage,damagetaken,healing,overhealing,healingtaken,activetime))
					petrecord.Touched = nil
				end
			end
		end
	end

	-- Sync self
	Recount:SendCommMessage("RECOUNT",serialdata,"WHISPER",target)
	-- Sync pets
	for i=1,#serialpetdata do
		Recount:SendCommMessage("RECOUNT",serialpetdata[i],"WHISPER",target) 
	end
end

function Recount:BroadcastLazySync()

	if not Recount.lazysync then return end -- Elsia: Nothing to lazy sync

	local _ , instanceType = IsInInstance()
	
	if instanceType == "pvp" then return end
	
	Recount.lazysync = false
	
	local validdata
	-- Prepare self
	local myname = Recount.PlayerName
	local combatant = Recount.db2.combatants[myname]
	
	if not combatant then
		return
	end
	local myrecord = combatant.Sync
	local serialdata
	if myrecord and myrecord.Touched then
		local damage = myrecord.Damage or 0
		local damagetaken = myrecord.DamageTaken or 0
		local healing = myrecord.Healing or 0
		local healingtaken = myrecord.HealingTaken or 0
		local overhealing = myrecord.Overhealing or 0
		local activetime = myrecord.ActiveTime or 0
		if  damage + damagetaken + healing + overhealing + healingtaken ~= 0 then
			serialdata = Recount:Serialize("PU",myname,damage,damagetaken,healing,overhealing,healingtaken,activetime)
			validdata = true
			myrecord.Touched = nil
		end
	end
	
	-- Prepare Pets
	local data=combatant	
	local serialpetdata = {}
	
	if data and data.Pet then
		for i=1,#data.Pet do
			local petrecord = Recount.db2.combatants[data.Pet[i]] and Recount.db2.combatants[data.Pet[i]].Sync
			if petrecord and petrecord.Touched then
				local damage = petrecord.Damage or 0
				local damagetaken = petrecord.DamageTaken or 0
				local healing = petrecord.Healing or 0
				local healingtaken = petrecord.HealingTaken or 0
				local overhealing = petrecord.Overhealing or 0
				local activetime = petrecord.ActiveTime or 0
				if  damage + damagetaken + healing + overhealing + healingtaken ~= 0 then
					tinsert(serialpetdata, Recount:Serialize("PU",data.Pet[i],damage,damagetaken,healing,overhealing,healingtaken,activetime))
					validdata = true
					petrecord.Touched = nil
				end
			end
		end
	end

	-- Prepare bosses
	local serialbossdata = {}
	
	local plevel = Recount.db2.combatants[Recount.PlayerName].level
	
	if not plevel then
		plevel = UnitLevel("player")
	end
	
	
	for k,v in pairs(Recount.db2.combatants) do
		if v.Sync and v.Sync.Touched and (v.type == "Boss" or (v.level and v.level > plevel)) then
			--Recount:Print("Boss: "..k)
			local myrecord = v.Sync
			local damage = myrecord.Damage or 0
			local damagetaken = myrecord.DamageTaken or 0
			local healing = myrecord.Healing or 0
			local healingtaken = myrecord.HealingTaken or 0
			local overhealing = myrecord.Overhealing or 0
			local activetime = myrecord.ActiveTime or 0
			if  damage + damagetaken + healing + overhealing + healingtaken ~= 0 then
				tinsert(serialbossdata, Recount:Serialize("PU",v.Name,damage,damagetaken,healing,overhealing,healingtaken,activetime))
			end
			validdata = true
			v.Sync.Touched = nil
		end
	end

	if not validdata then
		return
	end
	
	-- This is for testing. It'll whisper the player so you see syncs yourself.
--[[	local name = Recount.PlayerName
	local combatant = Recount.db2.combatants[name]
	if combatant and combatant.lazysync then
		-- Sync self
		if serialdata and myrecord.Damage + myrecord.DamageTaken + myrecord.Healing + myrecord.Overhealing + myrecord.HealingTaken + myrecord.ActiveTime ~= 0 then
			Recount:SendCommMessage("RECOUNT",serialdata,"WHISPER",name) 
		end

		-- Sync pets
		for i=1,#serialpetdata do
			--Recount:Print("Pet"..i..": "..serialpetdata[i])
			Recount:SendCommMessage("RECOUNT",serialpetdata[i],"WHISPER",name) 
		end
	
		-- Sync bosses
		for i=1,#serialbossdata do
			--Recount:Print("Boss"..i..": "..serialbossdata[i])
			Recount:SendCommMessage("RECOUNT",serialbossdata[i],"WHISPER",name)
		end
		
		-- Done syncing, thank you very much.
		
		combatant.lazysync = nil				
	end--]]

	if GetNumRaidMembers() > 0 then
		--local myname = Recount.PlayerName
		--local myrecord = Recount.db2.combatants[myname].Sync
		--local serialdata = Recount:Serialize("PU",myname,myrecord.Damage,myrecord.DamageTaken,myrecord.Healing,myrecord.Overhealing,myrecord.HealingTaken,myrecord.ActiveTime) -- Elsia: Shouldn't be necessary, we prep'd it.

		for i = 1, 40, 1 do --GetNumRaidMembers()
			if UnitExists("raid"..i) then
				local name = UnitName("raid"..i)
				
				if Recount.VerNum and Recount.VerNum[name] then -- Elsia: Only sync if we have a valid version
				
					local combatant = Recount.db2.combatants[name]
					if combatant and combatant.lazysync and name~= Recount.PlayerName and UnitIsConnected(name) then -- We don't sync with self
						-- Sync self
						if serialdata then
							Recount:SendCommMessage("RECOUNT",serialdata,"WHISPER",name)
						end

						-- Sync pets
						for i=1,#serialpetdata do
							Recount:SendCommMessage("RECOUNT",serialpetdata[i],"WHISPER",name) 
						end
					
						-- Sync bosses
						for i=1,#serialbossdata do
							Recount:SendCommMessage("RECOUNT",serialbossdata[i],"WHISPER",name)
						end
						combatant.lazysync = nil				
					end
				end
			end
		end
	elseif GetNumPartyMembers() > 0 then
		--local myname = Recount.PlayerName
		--local myrecord = Recount.db2.combatants[myname].Sync
		--local serialdata = Recount:Serialize("PU",myname,myrecord.Damage,myrecord.DamageTaken,myrecord.Healing,myrecord.Overhealing,myrecord.HealingTaken,myrecord.ActiveTime)

		for i = 1, 4, 1 do 
			if UnitExists("party"..i) then
				local name = UnitName("party"..i)
				if Recount.VerNum and Recount.VerNum[name] then -- Elsia: Only sync if we have a valid version
					local combatant = Recount.db2.combatants[name]
					if combatant and combatant.lazysync and name~= Recount.PlayerName and UnitIsConnected(name) then -- We don't sync with self
						-- Sync self
						if serialdata then
							Recount:SendCommMessage("RECOUNT",serialdata,"WHISPER",name) 
						end

						-- Sync pets
						for i=1,#serialpetdata do
							--Recount:Print("Pet"..i..": "..serialpetdata[i])
							Recount:SendCommMessage("RECOUNT",serialpetdata[i],"WHISPER",name) 
						end
					
						-- Sync bosses
						for i=1,#serialbossdata do
							--Recount:Print("Boss"..i..": "..serialbossdata[i])
							Recount:SendCommMessage("RECOUNT",serialbossdata[i],"WHISPER",name)
						end
						combatant.lazysync = nil				
					end
				end
			end
		end
	end
end

-- Lazy healing correction

function Recount:BroadcastHealCorrection()

	if not Recount.healingcorrection then return end

	local _ , instanceType = IsInInstance()
	
	if instanceType == "pvp" then return end
	
	Recount.healingcorrection = false
	
	if GetNumRaidMembers() > 0 then

		for i = 1, 40, 1 do --GetNumRaidMembers()
			if UnitExists("raid"..i) then
				local name = UnitName("raid"..i)
		
				local myrecord = Recount.db2.combatants[name] and Recount.db2.combatants[name].Sync
				if myrecord and  myrecord.HealingCorrection + myrecord.OverhealCorrection ~= 0 then
					Recount:SendCommMessage("RECOUNT",Recount:Serialize("HC",name,myrecord.HealingCorrection,myrecord.OverhealCorrection),"RAID")
					myrecord.HealingCorrection = 0
					myrecord.OverhealCorrection = 0
				end
			end
		end
	elseif GetNumPartyMembers() > 0 then

		for i = 1, 4, 1 do 
			if UnitExists("party"..i) then
				local name = UnitName("party"..i)

				local myrecord = Recount.db2.combatants[name] and Recount.db2.combatants[name].Sync
				if myrecord and myrecord.HealingCorrection + myrecord.OverhealCorrection ~= 0 then
					Recount:SendCommMessage("RECOUNT",Recount:Serialize("HC",name,myrecord.HealingCorrection,myrecord.OverhealCorrection),"RAID")
					myrecord.HealingCorrection = 0
					myrecord.OverhealCorrection = 0
				end
			end
		end
	end
end

function Recount:SendHealCorrection(name, heal, overheal) -- Elsia: Currently we are not correcting details.

	local _ , instanceType = IsInInstance()
	
	if instanceType == "pvp" then return end


	local combatant = Recount.db.char.combatants[name]
	
	if not combatant then
		Recount:Print("Healing correction. Uninitialized Unit: "..name)
		return
	end
	
	Recount.db.char.combatants[name].Sync = Recount.db.char.combatants[name].Sync or {}
	local combatant = Recount.db.char.combatants[name].Sync
	
	combatant.HealingCorrection = (combatant.HealingCorrection or 0) + heal
	combatant.OverhealCorrection = (combatant.OverhealCorrection or 0) + overheal
	
	Recount.healingcorrection = true
end



-- Elsia: Greater Elemental Syncing here.
-- TODO: First damage handling for non-owners

--[[local activeElementalName = {}

function Recount:AddGreaterElemental(petGUID,pet)
	local _ , instanceType = IsInInstance()
	
	if instanceType == "pvp" then return end

	activeElementalName[petGUID]=pet
	Recount:ScheduleTimer("CleanUpElemental", 120.0, petGUID) -- Elsia: After 2 mins the elemental will be toast regardless
end

function Recount:CleanUpElemental(petGUID)
	activeElementalName[petGUID]=nil
end

function Recount:ActiveElementalName(petGUID)
	return activeElementalName[petGUID]
end
]]
function Recount:OnCommReceive(prefix, Msgs, distribution, target)
	if distribution == "WHISPER" then
		local worked, cmd
		worked, cmd, name,damage,damageTaken,healing,overhealing,healingTaken,activeTime = Recount:Deserialize(Msgs)
		if worked == true then
			if cmd == "PU" then -- Player Update
				--Recount:Print(cmd .." "..name.." "..(damage or "nil").." "..(damageTaken or "nil").." "..(healing or "nil").." "..(overhealing or "nil").." "..(healingTaken or "nil").." "..(activeTime or "nil"))
				local combatant = Recount.db2.combatants[name]
				if not combatant then
					local nameFlags
					local owner = name:match("<(.-)>")
					if not owner then
						nameFlags = PARTY_GUARDIAN_OWNER_FLAGS
					else
						nameFlags = PARTY_PET_FLAGS
					end
					Recount:AddCombatant(name,owner,nil,nameFlags,nil)
					combatant = Recount.db2.combatants[name]
				end
				
				if Recount:CheckRetention(name) then
					local who = combatant
					combatant = combatant.Sync or {}
			
					if damage and combatant.Damage and damage > combatant.Damage then
						--Recount:Print("dmg: "..who.Name.." "..damage - combatant.Damage)
						Recount:AddAmount(who,"Damage",damage - combatant.Damage)
						combatant.Damage = damage
					end
					if damageTaken and combatant.DamageTaken and damageTaken > combatant.DamageTaken then
						Recount:AddAmount(who,"DamageTaken",damageTaken - combatant.DamageTaken)
						combatant.DamageTaken = damageTaken
					end
					if healing and combatant.Healing and healing > combatant.Healing then
						Recount:AddAmount(who,"Healing",healing - combatant.Healing)
						combatant.Healing = healing
					end
					if overhealing and combatant.Overhealing and overhealing > combatant.Overhealing then
						Recount:AddAmount(who,"Overhealing",overhealing - combatant.Overhealing)
						combatant.Overhealing = overhealing
					end
					if healingTaken and combatant.HealingTaken and healingTaken > combatant.HealingTaken then
						Recount:AddAmount(who,"HealingTaken",healingTaken - combatant.HealingTaken)
						combatant.HealingTaken = healingTaken
					end
					if activeTime and combatant.ActiveTime and activeTime > combatant.ActiveTime then
						Recount:AddAmount(who,"ActiveTime",activeTime - combatant.ActiveTime)
						combatant.ActiveTime = activeTime
					end
				end
			elseif cmd == "PS" then -- Player data set (when first meeting up)
				--Recount:Print(cmd .." "..name.." "..(damage or "nil").." "..(damageTaken or "nil").." "..(healing or "nil").." "..(overhealing or "nil").." "..(healingTaken or "nil").." "..(activeTime or "nil"))
				local combatant = Recount.db2.combatants[name]
				if not combatant then
					local nameFlags
					local owner = name:match("<(.-)>")
					if not owner then
						nameFlags = PARTY_GUARDIAN_OWNER_FLAGS
					else
						nameFlags = PARTY_PET_FLAGS
					end
					Recount:AddCombatant(name,owner,nil,nameFlags,nil)
					combatant = Recount.db2.combatants[name]
				end
				
				if Recount:CheckRetention(name) then
					local who = combatant
					combatant = combatant.Sync or {}
			
					Recount:SetSyncAmount(who,"Damage",damage)
					combatant.Damage = damage
					Recount:SetSyncAmount(who,"DamageTaken",damageTaken)
					combatant.DamageTaken = damageTaken
					Recount:SetSyncAmount(who,"Healing",healing)
					combatant.Healing = healing
					Recount:SetSyncAmount(who,"Overhealing",overhealing)
					combatant.Overhealing = overhealing
					Recount:SetSyncAmount(who,"HealingTaken",healingTaken)
					combatant.HealingTaken = healingTaken
					Recount:SetSyncAmount(who,"ActiveTime",activeTime)
					combatant.ActiveTime = activeTime
				end
			elseif cmd == "VS" then -- Version Whisper
				-- owner == originator, damage == version string :D
				local owner = name
				local version = damage
				if type(version)~="number" then
					Recount.VerNum[owner]=tonumber(string.match(version,"Revision: (%d+)")) -- Elsia: Old format
				else
					Recount.VerNum[owner] =version
				end
				if not Recount.VerNum[owner] or Recount.VerNum[owner]<MinimumVersion then
					Recount.VerTable[owner]="|cffff2020Incompatible|r "..version
				else
					Recount.VerTable[owner]=version
				end
			end
		end
	elseif distribution == "RAID" then
		local worked, cmd, owner, pet, petGUID
		worked, cmd, owner, pet, petGUID = Recount:Deserialize(Msgs)
		if worked == true then
--[[			if cmd == "AP" then -- Announce Pet GUID
				--Recount:Print(cmd.." "..owner.." "..pet.." "..petGUID)
				local name = pet.." <"..owner..">"
				Recount:AddPetCombatant(petGUID,pet, PARTY_GUARDIAN_FLAGS, UnitGUID(owner), owner, PARTY_GUARDIAN_OWNER_FLAGS)
				--Recount:AddGreaterElemental(petGUID,name)
			elseif]]
			if cmd == "RS" then -- Reset broadcast
				local combatant = Recount.db2.combatants[owner]
				if not combatant then
					Recount:AddCombatant(owner,nil,nil,COMBATLOG_OBJECT_AFFILIATION_RAID+COMBATLOG_OBJECT_TYPE_PLAYER+COMBATLOG_OBJECT_REACTION_FRIENDLY,nil)
					combatant = Recount.db2.combatants[owner]
				end
				if Recount:CheckRetention(name) then
					local who = combatant
					combatant = combatant.Sync or {}
					Recount:SetSyncAmount(who,"Damage",0)
					combatant.Damage = 0
					Recount:SetSyncAmount(who,"DamageTaken",0)
					combatant.DamageTaken = 0
					Recount:SetSyncAmount(who,"Healing",0)
					combatant.Healing = 0
					Recount:SetSyncAmount(who,"Overhealing",0)
					combatant.Overhealing = 0
					Recount:SetSyncAmount(who,"HealingTaken",0)
					combatant.HealingTaken = 0
					Recount:SetSyncAmount(who,"ActiveTime",0)
					combatant.ActiveTime = 0
				end
			elseif cmd == "VS" or cmd == "VQ" then -- Version Broadcast
				-- owner == originator, pet == version string :D
				local version = pet
				--Recount:Print(cmd.." "..owner.." "..version)
				if type(version)~="number" then
					Recount.VerNum[owner]=tonumber(string.match(version,"Revision: (%d+)")) -- Elsia: Old format
				else
					Recount.VerNum[owner] =version
				end
				if not Recount.VerNum[owner] or Recount.VerNum[owner]<MinimumVersion then
					Recount.VerTable[owner]="|cffff2020Incompatible|r "..version
				else
					Recount.VerTable[owner]=version
				end
				if cmd == "VQ" then
					Recount:SendVersion(owner) -- Return own version if requested.
				end
			elseif cmd == "HC" then -- Healing correction
				local name = owner
				local healing = pet
				local overhealing = petGUID
				local combatant = Recount.db2.combatants[name]
				if not combatant then
					-- Elsia: folks who haven't even seen the combatant don't need to be corrected. They need a lazy sync first...
					return
				end
				
				local who = combatant
				combatant = combatant.Sync or {}
		
				if healing and combatant.Healing then
					Recount:AddAmount(who,"Healing", -healing)
					combatant.Healing = combatant.Healing-healing
				end
				if overhealing and combatant.Overhealing then
					Recount:AddAmount(who,"Overhealing", -overhealing)
					combatant.Overhealing = combatant.Overhealing-overhealing
				end
			end
		end
	end
end

function Recount:ConfigComm()
	Recount.VerTable={}
	Recount.VerNum={}

	Recount:RegisterComm("RECOUNT", "OnCommReceive")	
	Recount:RequestVersion()
end

function Recount:FreeComm()
	Recount:UnregisterComm("RECOUNT")
end

--[[function Recount:AnnouncePetGUID(owner,pet,petGUID)
	Recount:SendCommMessage("RECOUNT",Recount:Serialize("AP",owner,pet,petGUID),"RAID")
	Recount:FlagSync() -- Elsia: Make sure to sync possibly missed early damage.
end]]

function Recount:FlagSync()
	if GetNumRaidMembers() > 0 then
		for i = 1, 40, 1 do 
			if UnitExists("raid"..i) then
				Recount.db2.combatants[UnitName("raid"..i)].lazysync = true
				Recount.lazysync = true
			end
		end
	elseif GetNumPartyMembers() > 0 then
		for i = 1, 4, 1 do 
			if UnitExists("party"..i) then
				Recount.db2.combatants[UnitName("party"..i)].lazysync = true
				Recount.lazysync = true
			end
		end
	end
end

function Recount:SendReset()
	Recount:SendCommMessage("RECOUNT",Recount:Serialize("RS",Recount.PlayerName),"RAID")
end

function Recount:SendVersion(target)
	if target then
		if UnitIsConnected(target) then
			Recount:SendCommMessage("RECOUNT",Recount:Serialize("VS",Recount.PlayerName,Recount.Version),"WHISPER",target)
			Recount:SendSelf(target)
		end
	else
		Recount:SendCommMessage("RECOUNT",Recount:Serialize("VS",Recount.PlayerName,Recount.Version),"RAID")
	end
end

function Recount:RequestVersion()
	Recount:SendCommMessage("RECOUNT",Recount:Serialize("VQ",Recount.PlayerName,Recount.Version),"RAID")
end
